@@ -19,6 +19,7 @@
 #include "libstr.h"
 #include "send.h"
 #include "cgi.h"
+#include "client.h"
 #include "serverconfig.h"
 #include "envir.h"
 #include "log.h"
@@ -269,7 +270,7 @@ t_fcgi_server *find_fcgi_server(t_fcgi_server *fcgi_server, char *id) {
 
 pid_t fork_cgi_process(t_session *session, t_cgi_info *cgi_info) {
 	int post_pipe[2], html_pipe[2], error_pipe[2], i;
-	char *pos, slash = '/', *run[7], cgi_time[16];
+	char *pos, slash = '/', *run[10], cgi_time[16];
 	pid_t cgi_pid;
 
 	do {
@@ -310,9 +311,9 @@ pid_t fork_cgi_process(t_session *session, t_cgi_info *cgi_info) {
 
 			/* Close all other open filedescriptors.
 			 */
-			close(session->client_socket);
 			close_bindings(session->config->binding);
-			log_close_for_cgi_run(session->config->first_host);
+			close_client_sockets_for_cgi_run();
+			close_logfiles_for_cgi_run(session->config->first_host);
 
 			set_environment(session, NULL);
 
