@@ -190,32 +190,6 @@ int remove_client(t_session *session, bool free_session) {
 	}
 }
 
-/* Check if a write has blocked for too long
- */
-/*
-void check_write_blocks_on_socket(int max_write_block_time) {
-	t_client *client;
-	int i;
-
-	for (i = 0; i < 256; i++) {
-		pthread_mutex_lock(&client_mutex[i]);
-
-		client = client_list[i];
-		while (client != NULL) {
-			if (client->session->socket_open && (client->session->send_timer >= 0)) {
-				if ((client->session->send_timer)++ == max_write_block_time) {
-					close_socket(client->session);
-				}
-			}
-
-			client = client->next;
-		}
-	
-		pthread_mutex_unlock(&client_mutex[i]);
-	}
-}
-*/
-
 /* Check whether to allow or deny a new connection.
  */
 int connection_allowed(t_ip_addr *ip, int maxperip, int maxtotal) {
@@ -559,6 +533,8 @@ int register_wrong_password(t_session *session) {
 	return 0;
 }
 
+/* Remove all clients from the wrong-password list
+ */
 void remove_wrong_password_list(t_config *config) {
 	t_ipcounterlist *item, *remove;
 
@@ -585,6 +561,21 @@ void remove_wrong_password_list(t_config *config) {
 	delay_timer = 0;
 }
 
+/* Close all client sockets to run a CGI program
+ */
+void close_client_sockets_for_cgi_run(void) {
+	t_client *client;
+	int i;
+
+	for (i = 0; i < 256; i++) {
+		client = client_list[i];
+		while (client != NULL) {
+			close(client->session->client_socket);
+			client = client->next;
+		}
+	}
+}
+
 #ifdef HAVE_COMMAND
 /* Print the list of current connections.
  */
