@@ -364,7 +364,9 @@ int handle_admin(t_admin *admin, t_config *config) {
 		admin->timer = MAX_IDLE_TIME;
 		if (strlen(line) >= MAX_CMD_SIZE - 1) {
 			do {
-				fgets(line, MAX_CMD_SIZE, admin->fp);
+				if (fgets(line, MAX_CMD_SIZE, admin->fp) == NULL) {
+					return cc_DISCONNECT;
+				}
 			} while (strlen(line) >= MAX_CMD_SIZE - 1);
 			if (admin->authenticated == false) {
 				fprintf(admin->fp, "\033[00m");
