@@ -155,6 +155,7 @@ void touch_logfiles(t_config *config, char *dir) {
 t_cgi_type check_target_is_cgi(t_session *session) {
 	t_cgi_handler *cgi;
 
+	session->cgi_handler = NULL;
 #ifdef HAVE_TOOLKIT
 	if ((session->fcgi_server = find_fcgi_server(session->config->fcgi_server, session->toolkit_fastcgi)) != NULL) {
 		session->cgi_type = fastcgi;
@@ -165,6 +166,7 @@ t_cgi_type check_target_is_cgi(t_session *session) {
 	} else if (in_charlist(session->extension, &(session->config->cgi_extension))) {
 		session->cgi_type = binary;
 	} else {
+		session->cgi_type = no_cgi;
 		cgi = session->config->cgi_handler;
 		while (cgi != NULL) {
 			if (in_charlist(session->extension, &(cgi->extension))) {
@@ -183,7 +185,7 @@ t_cgi_type check_target_is_cgi(t_session *session) {
  */
 int handle_error(t_session *session, int error_code) {
 	char *error_handler = NULL;
-	int length, result = -1;
+	int result = -1;
 
 	switch (error_code) {
 		case 401:
@@ -213,17 +215,15 @@ int handle_error(t_session *session, int error_code) {
 	if (session->file_on_disk != NULL) {
 		free(session->file_on_disk);
 	}
-	length = session->host->website_root_len;
-	if ((session->file_on_disk = (char*)malloc(length + strlen(error_handler) + 4)) == NULL) { /* + 3 for .gz (gzip encoding) */
+	if ((session->file_on_disk = (char*)malloc(session->host->website_root_len + strlen(error_handler) + 4)) == NULL) { /* + 3 for .gz (gzip encoding) */
 		result = 500;
 	} else {
-		memcpy(session->file_on_disk, session->host->website_root, length + 1);
-		strcpy(session->file_on_disk + length, error_handler);
+		memcpy(session->file_on_disk, session->host->website_root, session->host->website_root_len);
+		strcpy(session->file_on_disk + session->host->website_root_len, error_handler);
 
 		if (get_target_extension(session) == -1) {
 			return 500;
 		}
-		session->cgi_type = no_cgi;
 		check_target_is_cgi(session);
 
 		if (session->cgi_type != no_cgi) {
@@ -273,34 +273,64 @@ int handle_error(t_session *session, int error_code) {
 	return result;
 }
 
-/* Run RunOnAlter script after PUT or DELETE
+/* Run a program
  */
-int run_alter_program(t_session *session) {
+int run_program(t_session *session, char *program) {
 	pid_t pid;
-	char ip[MAX_IP_STR_LEN];
+	char ip[MAX_IP_STR_LEN], *pos, slash = '/';
+	int result;
 
-	if ((pid = fork()) == 0) {
-		if (setsid() != -1) {
-			setenv("REQUEST_METHOD", session->method, 1);
-			setenv("DOCUMENT_ROOT", session->host->website_root, 1);
-			setenv("REQUEST_URI", session->request_uri, 1);
-			if (session->remote_user != NULL) {
-				setenv("REMOTE_USER", session->remote_user, 1);
+	switch (pid = fork()) {
+		case -1:
+			log_file_error(session, program, "fork() error");
+			return -1;
+		case 0:
+			if (setsid() == -1) {
+				log_file_error(session, program, "setsid() error");
+			} else {
+				/* Close all other open filedescriptors.
+				 */
+				close_bindings(session->config->binding);
+				close_client_sockets_for_cgi_run();
+				close_logfiles_for_cgi_run(session->config->first_host);
+
+				/* Set environment variables
+				 */
+				setenv("REQUEST_METHOD", session->method, 1);
+				setenv("DOCUMENT_ROOT", session->host->website_root, 1);
+				setenv("REQUEST_URI", session->request_uri, 1);
+				if (session->remote_user != NULL) {
+					setenv("REMOTE_USER", session->remote_user, 1);
+				}
+				if (inet_ntop(session->ip_address.family, &(session->ip_address.value), ip, MAX_IP_STR_LEN) != NULL) {
+					setenv("REMOTE_ADDR", ip, 1);
+				}
+
+				/* Change directory to program's directory
+				 */
+				pos = strrchr(program, slash);
+#ifdef CYGWIN
+				if ((pos == NULL) && (session->config->platform == windows)) {
+					slash = '\\';
+					pos = strrchr(program, slash);
+				}
+#endif
+				if (pos != NULL) {
+					*pos = '\0';
+					result = chdir(program);
+					*pos = slash;
+				}
+
+				/* Execute program
+				 */
+				execlp(program, program, (char*)NULL);
+				log_file_error(session, program, "exec() error");
 			}
-			if (inet_ntop(session->ip_address.family, &(session->ip_address.value), ip, MAX_IP_STR_LEN) != NULL) {
-				setenv("REMOTE_ADDR", ip, 1);
+			exit(EXIT_FAILURE);
+		default:
+			if (session->config->wait_for_cgi) {
+				waitpid(pid, NULL, 0);
 			}
-			execlp(session->host->run_on_alter, session->host->run_on_alter, (char*)NULL);
-			log_file_error(session, session->host->run_on_alter, "couldn't execute RunOnAlter");
-		} else {
-			log_file_error(session, session->host->run_on_alter, "RunOnAlter setsid() error");
-		}
-		exit(EXIT_FAILURE);
-	} else if (pid == -1) {
-		log_file_error(session, session->host->run_on_alter, "RunOnAlter fork() error");
-		return -1;
-	} else if (session->config->wait_for_cgi) {
-		waitpid(pid, NULL, 0);
 	}
 
 	return 0;
@@ -707,7 +737,7 @@ int uri_to_path(t_session *session) {
 	} else {
 		length = strlen(alias->value);
 	}
-	length += session->uri_len + strlen(session->host->start_file);
+	length += session->uri_len + MAX_START_FILE_LENGTH;
 	if ((session->file_on_disk = (char*)malloc(length + 4)) == NULL) { /* + 3 for '.gz' (gzip encoding) */
 		return 500;
 	}
@@ -1004,6 +1034,11 @@ int serve_client(t_session *session) {
 
 	switch (session->request_method) {
 		case GET:
+			if ((session->cgi_type == no_cgi) && (session->directory != NULL)) {
+				if (session->directory->run_on_download != NULL) {
+					run_program(session, session->directory->run_on_download);
+				}
+			}
 		case HEAD:
 			if (session->cgi_type != no_cgi) {
 				session->body = NULL;
@@ -1038,13 +1073,13 @@ int serve_client(t_session *session) {
 		case PUT:
 			result = handle_put_request(session);
 			if (((result == 201) || (result == 204)) && (session->host->run_on_alter != NULL)) {
-				run_alter_program(session);
+				run_program(session, session->host->run_on_alter);
 			}
 			break;
 		case DELETE:
 			result = handle_delete_request(session);
 			if ((result == 204) && (session->host->run_on_alter != NULL)) {
-				run_alter_program(session);
+				run_program(session, session->host->run_on_alter);
 			}
 			break;
 		default:
@@ -1244,9 +1279,9 @@ void task_runner(t_config *config) {
 		 */
 		check_load_balancer(config);
 
-		/* AccessLogfile check
+		/* Close idle logfile handles
 		 */
-		log_close(config->first_host, false);
+		close_logfiles(config->first_host, false);
 
 #ifdef HAVE_CACHE
 		/* Cache check
@@ -1276,7 +1311,7 @@ void task_runner(t_config *config) {
 				received_signal = rs_NONE;
 				break;
 			case rs_UNLOCK_LOGFILES:
-				log_close(config->first_host, true);
+				close_logfiles(config->first_host, true);
 				received_signal = rs_NONE;
 				break;
 #ifdef HAVE_CACHE
@@ -1596,7 +1631,7 @@ int run_server(t_settings *settings) {
 	} else if (settings->config_check) {
 		printf("Using %s\n", settings->config_dir);
 	}
-	if (read_main_configfile("httpd.conf", config, settings->config_check) == -1) {
+	if (read_main_configfile("hiawatha.conf", config, settings->config_check) == -1) {
 		return EXIT_FAILURE;
 	} else if (check_configuration(config) == -1) {
 		return EXIT_FAILURE;
@@ -1928,7 +1963,7 @@ int run_server(t_settings *settings) {
 #endif
 
 	log_string(config->system_logfile, "Hiawatha v"VERSION" stopped");
-	log_close(config->first_host, true);
+	close_logfiles(config->first_host, true);
 
 	return EXIT_SUCCESS;
 }
