@@ -7,6 +7,7 @@
 #include <sys/stat.h>
 #include <stdio.h>
 #include <stdbool.h>
+#include <unistd.h>
 #include <string.h>
 #include <time.h>
 #include <pthread.h>
@@ -82,9 +83,12 @@ void log_pid(t_config *config, pid_t pid) {
 	if ((fp = fopen(config->pidfile, "w")) != NULL) {
 		fprintf(fp, "%d\n", (int)pid);
 		fclose(fp);
-		if (chmod(config->pidfile, S_IRUSR | S_IWUSR | S_IRGRP) == -1) {
+		if (chmod(config->pidfile, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH) == -1) {
 			fprintf(stderr, "Warning: can't chmod PID file %s. Make sure it's only writable for root!\n", config->pidfile);
 		}
+		if (chown(config->pidfile, 0, 0) == -1) {
+			fprintf(stderr, "Warning: can't chown PID file %s. Make sure it's owned by root!\n", config->pidfile);
+		}
 	} else {
 		fprintf(stderr, "Warning: can't write PID file %s.\n", config->pidfile);
 	}
@@ -129,7 +133,7 @@ void log_file_error(t_session *session, char *file, char *mesg) {
 		if (file != NULL) {
 			fprintf(fp, "%s%s|%s"EOL, str, file, mesg);
 		} else {
-			fprintf(fp, "%s%s"EOL, str, mesg);
+			fprintf(fp, "%s-|%s"EOL, str, mesg);
 		}
 		fclose(fp);
 	}
@@ -169,21 +173,22 @@ void log_request(t_session *session, int code) {
 		}
 #endif
 
-		if ((user = session->remote_user) == NULL) {
-			user = "";
-		} else {
+		if ((user = session->remote_user) != NULL) {
 			user = secure_string(user);
 		}
 
 		if (session->config->log_format == hiawatha) {
 			/* Hiawatha log format
 			 */
-
 			print_ip_address(str, &(session->ip_address));
 			offset = strlen(str);
 			print_timestamp(str + offset);
 			offset += strlen(str + offset);
 
+			if (user == NULL) {
+				user = "";
+			}
+
 			snprintf(str + offset, BUFFER_SIZE - offset, "%d|%lld|%s|%s %s", code, (long long)session->bytes_sent, user, secure_string(session->method), uri);
 			offset += strlen(str + offset);
 
@@ -221,6 +226,10 @@ void log_request(t_session *session, int code) {
 				ip_address[len - 1] = '\0';
 			}
 
+			if (user == NULL) {
+				user = "-";
+			}
+
 			time(&t);
 			s = localtime(&t);
 			timestamp[TIMESTAMP_SIZE - 1] = '\0';
@@ -248,7 +257,7 @@ void log_request(t_session *session, int code) {
 					if ((field = get_headerfield("Referer:", session->headerfields)) != NULL) {
 						snprintf(str + offset, BUFFER_SIZE - offset, " \"%s\"", secure_string(field));
 					} else {
-						snprintf(str + offset, BUFFER_SIZE - offset, " -");
+						snprintf(str + offset, BUFFER_SIZE - offset, " \"-\"");
 					}
 					offset += strlen(str + offset);
 				}
@@ -256,7 +265,7 @@ void log_request(t_session *session, int code) {
 					if ((field = get_headerfield("User-Agent:", session->headerfields)) != NULL) {
 						snprintf(str + offset, BUFFER_SIZE - offset, " \"%s\"", secure_string(field));
 					} else {
-						snprintf(str + offset, BUFFER_SIZE - offset, " -");
+						snprintf(str + offset, BUFFER_SIZE - offset, " \"-\"");
 					}
 				}
 			}
@@ -341,7 +350,11 @@ void log_cgi_error(t_session *session, char *mesg) {
 		if ((fp = fopen(session->host->error_logfile, "a")) != NULL) {
 			print_timestamp(str);
 			print_ip_address(str + strlen(str), &(session->ip_address));
-			fprintf(fp, "%s%s|%s"EOL, str, session->file_on_disk, secure_string(mesg));
+			if (session->file_on_disk == NULL) {
+				fprintf(fp, "%s-|%s"EOL, str, secure_string(mesg));
+			} else {
+				fprintf(fp, "%s%s|%s"EOL, str, session->file_on_disk, secure_string(mesg));
+			}
 			fclose(fp);
 		}
 	}
@@ -349,7 +362,7 @@ void log_cgi_error(t_session *session, char *mesg) {
 
 /* Close open access logfiles.
  */
-void log_close(t_host *host, bool force_close) {
+void close_logfiles(t_host *host, bool force_close) {
 	time_t now;
 
 	if ((++delay_timer >= MAX_LOG_DELAY_TIMER) || force_close) {
@@ -370,7 +383,7 @@ void log_close(t_host *host, bool force_close) {
 	}
 }
 
-void log_close_for_cgi_run(t_host *host) {
+void close_logfiles_for_cgi_run(t_host *host) {
 	while (host != NULL) {
 		if (*(host->access_fp) != NULL) {
 			fclose(*(host->access_fp));
