@@ -15,7 +15,7 @@ void log_request(t_session *session, int code);
 void log_garbage(t_session *session);
 void log_unban(char *logfile, t_ip_addr *ip_address, unsigned long connect_attempts);
 void log_cgi_error(t_session *session, char *mesg);
-void log_close(t_host *host, bool force_close);
-void log_close_for_cgi_run(t_host *host);
+void close_logfiles(t_host *host, bool force_close);
+void close_logfiles_for_cgi_run(t_host *host);
 
 #endif
