@@ -20,6 +20,6 @@ typedef struct type_mime {
 } t_mimetype;
 
 int  read_mimetypes(char *configfile, t_mimetype **mime);
-char *get_mimetype(char *filename, t_mimetype *mime);
+char *get_mimetype(char *extension, t_mimetype *mime);
 
 #endif
