@@ -18,6 +18,7 @@
 #endif
 #include "libstr.h"
 #include "libmd5.h"
+#include "log.h"
 #include "send.h"
 #ifdef HAVE_COMMAND
 #include "command.h"
@@ -26,6 +27,7 @@
 #define MAX_CHUNK_SIZE 2048
 #define MAX_TO_BUFFER 400
 #define NONCE_DIGITS 10
+#define SEND_TIMEOUT 5
 
 char *hs_http10 = "HTTP/1.0 ";                  /*  9 */
 char *hs_http11 = "HTTP/1.1 ";                  /*  9 */
@@ -83,7 +85,7 @@ static int send_to_client(t_session *session, const char *buffer, int size) {
 		} else {
 			can_send = rest;
 		}
-		
+
 #ifdef DEBUG
 	    set_client_status(session, "send_to_client()::send()");
 #endif
@@ -98,12 +100,20 @@ static int send_to_client(t_session *session, const char *buffer, int size) {
 			if ((bytes_sent = send(session->client_socket, (char*)(buffer + total_sent), can_send, 0)) <= 0) {
 				bytes_sent = -1;
 			}
+
 #ifdef DEBUG
 	    set_client_status(session, NULL);
 #endif
 
+		/* Handle read result
+		 */
 		if (bytes_sent == -1) {
 			if (errno != EINTR) {
+				if ((errno == EAGAIN) || (errno == EWOULDBLOCK)) {
+					log_error(session, "send timeout");
+				} else if ((errno != EPIPE) && (errno != ECONNRESET)) {
+					log_error(session, "socket error");
+				}
 				close_socket(session);
 				session->keep_alive = false;
 				return -1;
