@@ -133,6 +133,7 @@ static t_directory *new_directory(void) {
 		directory->session_speed       = 0;
 		directory->envir_str           = NULL;
 		directory->time_for_cgi        = -1;
+		directory->run_on_download     = NULL;
 		pthread_mutex_init(&(directory->client_mutex), NULL);
 
 		directory->next                = NULL;
@@ -284,7 +285,7 @@ t_config *default_config(void) {
 #ifdef HAVE_CACHE
 		config->cache_size         = 10 * MEGABYTE;
 		config->cache_max_filesize = 256 * KILOBYTE;
-		config->cache_min_filesize = 0;
+		config->cache_min_filesize = 1;
 #endif
 
 #ifdef HAVE_COMMAND
@@ -326,12 +327,12 @@ static int fgets_multi(char *line, int size, FILE *fp) {
 	}
 }
 
-static bool valid_file(char *file) {
+static bool valid_start_file(char *file) {
 	bool retval = false;
 
 	if (file != NULL) {
 		if (strchr(file, '/') == NULL) {
-			if (strlen(file) < 32) {
+			if (strlen(file) <= MAX_START_FILE_LENGTH) {
 				retval = true;
 			}
 		}
@@ -697,7 +698,7 @@ static bool system_setting(char *key, char *value, t_config *config) {
 			return true;
 		}
 	} else if (strcmp(key, "cacheminfilesize") == 0) {
-		if ((config->cache_min_filesize = str2int(value)) != -1) {
+		if ((config->cache_min_filesize = str2int(value)) > 0) {
 			return true;
 		}
 	} else if (strcmp(key, "cachesize") == 0) {
@@ -1045,13 +1046,13 @@ static bool user_setting(char *key, char *value, t_host *host, t_tempdata **temp
 			return true;
 		}
 	} else if (strcmp(key, "startfile") == 0) {
-		if (valid_file(value)) {
-			if ((host->start_file = strdup(value)) != NULL) {
-				if (register_tempdata(tempdata, host->start_file, tc_data) != -1) {
+		if (valid_start_file(value)) {
+			if ((value = strdup(value)) != NULL) {
+				if (register_tempdata(tempdata, value, tc_data) != -1) {
+					host->start_file = value;
 					return true;
 				} else {
-					free(host->start_file);
-					host->start_file = NULL;
+					free(value);
 				}
 			}
 		}
@@ -1278,6 +1279,10 @@ static bool directory_setting(char *key, char *value, t_directory *directory) {
 		if (parse_charlist(value, &(directory->required_group)) != -1) {
 			return true;
 		}
+	} else if (strcmp(key, "runondownload") == 0) {
+		if ((directory->run_on_download = strdup(value)) != NULL) {
+			return true;
+		}
 	} else if (strcmp(key, "setenv") == 0) {
 		if (parse_keyvaluelist(value, &(directory->envir_str), "=") != -1) {
 			return true;
@@ -1288,7 +1293,7 @@ static bool directory_setting(char *key, char *value, t_directory *directory) {
 			return true;
 		}
 	} else if (strcmp(key, "startfile") == 0) {
-		if (valid_file(value)) {
+		if (valid_start_file(value)) {
 			if ((directory->start_file = strdup(value)) != NULL) {
 				return true;
 			}
@@ -1897,25 +1902,26 @@ unsigned short get_throttlespeed(char *type, t_throttle *throttle) {
 	int len_type, len_throt;
 	char *type_lower;
 
-	if (type != NULL) {
-		if ((type_lower = strlower(strdup(type))) != NULL) {
-			len_type = strlen(type);
-
-			throt = throttle;
-			while (throt != NULL) {
-				len_throt = strlen(throt->filetype);
-				if (len_type >= len_throt) {
-					if (memcmp(throt->filetype, type_lower, len_throt) == 0) {
-						speed = throt->upload_speed;
-						break;
-					}
-				}
-				throt = throt->next;
-			}
-			free(type_lower);
-		}
+	if (type == NULL) {
+		return 0;
+	} else if ((type_lower = strlower(strdup(type))) == NULL) {
+		return 0;
 	}
 
+	len_type = strlen(type);
+	throt = throttle;
+	while (throt != NULL) {
+		len_throt = strlen(throt->filetype);
+		if (len_type >= len_throt) {
+			if (memcmp(throt->filetype, type_lower, len_throt) == 0) {
+				speed = throt->upload_speed;
+				break;
+			}
+		}
+		throt = throt->next;
+	}
+	free(type_lower);
+
 	return speed;
 }
 
