@@ -16,6 +16,9 @@
 #include "libssl.h"
 #endif
 
+#define MAX_START_FILE_LENGTH 32
+
+
 typedef enum { root, part } t_pathmatch;
 typedef enum { no_auth, basic, digest } t_auth_method;
 typedef enum { hiawatha, common, extended } t_log_format;
@@ -117,6 +120,7 @@ typedef struct type_directory {
 	char          *imgref_replacement;
 	t_keyvalue    *envir_str;
 	int           time_for_cgi;
+	char          *run_on_download;
 
 	/* Uploadspeed control
 	 */
