@@ -208,11 +208,16 @@ void determine_request_method(t_session *session) {
 /* Get the extension of the file to be processed
  */
 int get_target_extension(t_session *session) {
+	char *last_slash;
 #ifdef CIFS
 	sfree(session->extension);
 #endif
 
-	if ((session->extension = strrchr(session->file_on_disk, '.')) != NULL) {
+	if ((last_slash = strrchr(session->file_on_disk, '/')) == NULL) {	
+		return -1;
+	}
+
+	if ((session->extension = strrchr(last_slash, '.')) != NULL) {
 		(session->extension)++;
 	} else {
 		session->extension = session->config->no_extension_as;
