@@ -89,7 +89,7 @@ int send_file(t_session *session) {
 
 	/* Check the referer
 	 */
-	if (session->host->image_referer.size > 0) {
+	if ((session->host->image_referer.size > 0) && (session->mimetype != NULL)) {
 		if (strncmp(session->mimetype, "image/", 6) == 0) {
 			invalid_referer = true;
 			if ((referer = get_headerfield("Referer:", session->headerfields)) != NULL) {
