@@ -320,7 +320,7 @@ int check_main_config(char *config_dir) {
 	/* Read the configuration file
 	 */
 	config = NULL;
-	if (read_config_file(strdup("httpd.conf"), &config) != 0) {
+	if (read_config_file(strdup("hiawatha.conf"), &config) != 0) {
 		return 1;
 	}
 
@@ -561,7 +561,7 @@ void check_url_toolkit(char *config_dir, char **toolkit_id) {
 	/* Read the configuration file
 	 */
 	config = NULL;
-	if (read_config_file(strdup("httpd.conf"), &config) != 0) {
+	if (read_config_file(strdup("hiawatha.conf"), &config) != 0) {
 		return;
 	}
 
